@@ -1,10 +1,11 @@
 //! Defines types to use with the streams commands.
 
 use crate::{
-    from_redis_value, types::HashMap, FromRedisValue, RedisResult, RedisWrite, ToRedisArgs, Value,
+    from_redis_value, types::HashMap, ErrorKind, FromRedisValue, RedisError, RedisResult,
+    RedisWrite, ToRedisArgs, Value,
 };
 
-use std::io::{Error, ErrorKind};
+use std::io::Error;
 
 macro_rules! invalid_type_error {
     ($v:expr, $det:expr) => {{
@@ -44,6 +45,185 @@ impl ToRedisArgs for StreamMaxlen {
     }
 }
 
+/// Utility enum for passing the trim mode`[=|~]`
+/// arguments into `StreamCommands`.
+#[derive(Debug)]
+pub enum StreamTrimmingMode {
+    /// Match an exact count
+    Exact,
+    /// Match an approximate count
+    Approx,
+}
+
+impl ToRedisArgs for StreamTrimmingMode {
+    fn write_redis_args<W>(&self, out: &mut W)
+    where
+        W: ?Sized + RedisWrite,
+    {
+        match self {
+            Self::Exact => out.write_arg(b"="),
+            Self::Approx => out.write_arg(b"~"),
+        };
+    }
+}
+
+/// Utility enum for passing `<MAXLEN|MINID> [=|~] threshold [LIMIT count]`
+/// arguments into `StreamCommands`.
+/// The enum values the trimming mode (=|~), the threshold, and the optional limit
+#[derive(Debug)]
+pub enum StreamTrimStrategy {
+    /// Evicts entries as long as the streams length exceeds threshold.  With an optional limit.
+    MaxLen(StreamTrimmingMode, usize, Option<usize>),
+    /// Evicts entries with IDs lower than threshold, where threshold is a stream ID With an optional limit.
+    MinId(StreamTrimmingMode, String, Option<usize>),
+}
+
+impl StreamTrimStrategy {
+    /// Define a MAXLEN trim strategy with the given maximum number of entries
+    pub fn maxlen(trim: StreamTrimmingMode, max_entries: usize) -> Self {
+        Self::MaxLen(trim, max_entries, None)
+    }
+
+    /// Defines a MINID trim strategy with the given minimum stream ID
+    pub fn minid(trim: StreamTrimmingMode, stream_id: impl Into<String>) -> Self {
+        Self::MinId(trim, stream_id.into(), None)
+    }
+
+    /// Set a limit to the number of records to trim in a single operation
+    pub fn limit(self, limit: usize) -> Self {
+        match self {
+            StreamTrimStrategy::MaxLen(m, t, _) => StreamTrimStrategy::MaxLen(m, t, Some(limit)),
+            StreamTrimStrategy::MinId(m, t, _) => StreamTrimStrategy::MinId(m, t, Some(limit)),
+        }
+    }
+}
+
+impl ToRedisArgs for StreamTrimStrategy {
+    fn write_redis_args<W>(&self, out: &mut W)
+    where
+        W: ?Sized + RedisWrite,
+    {
+        let limit = match self {
+            StreamTrimStrategy::MaxLen(m, t, limit) => {
+                out.write_arg(b"MAXLEN");
+                m.write_redis_args(out);
+                t.write_redis_args(out);
+                limit
+            }
+            StreamTrimStrategy::MinId(m, t, limit) => {
+                out.write_arg(b"MINID");
+                m.write_redis_args(out);
+                t.write_redis_args(out);
+                limit
+            }
+        };
+        if let Some(limit) = limit {
+            out.write_arg(b"LIMIT");
+            limit.write_redis_args(out);
+        }
+    }
+}
+
+/// Builder options for [`xtrim_options`] command
+///
+/// [`xtrim_options`]: ../trait.Commands.html#method.xtrim_options
+///
+#[derive(Debug)]
+pub struct StreamTrimOptions {
+    strategy: StreamTrimStrategy,
+    deletion_policy: Option<StreamDeletionPolicy>,
+}
+
+impl StreamTrimOptions {
+    /// Define a MAXLEN trim strategy with the given maximum number of entries
+    pub fn maxlen(mode: StreamTrimmingMode, max_entries: usize) -> Self {
+        Self {
+            strategy: StreamTrimStrategy::maxlen(mode, max_entries),
+            deletion_policy: None,
+        }
+    }
+
+    /// Defines a MINID trim strategy with the given minimum stream ID
+    pub fn minid(mode: StreamTrimmingMode, stream_id: impl Into<String>) -> Self {
+        Self {
+            strategy: StreamTrimStrategy::minid(mode, stream_id),
+            deletion_policy: None,
+        }
+    }
+
+    /// Set a limit to the number of records to trim in a single operation
+    pub fn limit(mut self, limit: usize) -> Self {
+        self.strategy = self.strategy.limit(limit);
+        self
+    }
+
+    /// Set the deletion policy for the XTRIM operation
+    pub fn set_deletion_policy(mut self, deletion_policy: StreamDeletionPolicy) -> Self {
+        self.deletion_policy = Some(deletion_policy);
+        self
+    }
+}
+
+impl ToRedisArgs for StreamTrimOptions {
+    fn write_redis_args<W>(&self, out: &mut W)
+    where
+        W: ?Sized + RedisWrite,
+    {
+        self.strategy.write_redis_args(out);
+        if let Some(deletion_policy) = self.deletion_policy.as_ref() {
+            deletion_policy.write_redis_args(out);
+        }
+    }
+}
+
+/// Builder options for [`xadd_options`] command
+///
+/// [`xadd_options`]: ../trait.Commands.html#method.xadd_options
+///
+#[derive(Default, Debug)]
+pub struct StreamAddOptions {
+    nomkstream: bool,
+    trim: Option<StreamTrimStrategy>,
+    deletion_policy: Option<StreamDeletionPolicy>,
+}
+
+impl StreamAddOptions {
+    /// Set the NOMKSTREAM flag on which prevents creating a stream for the XADD operation
+    pub fn nomkstream(mut self) -> Self {
+        self.nomkstream = true;
+        self
+    }
+
+    /// Enable trimming when adding using the given trim strategy
+    pub fn trim(mut self, trim: StreamTrimStrategy) -> Self {
+        self.trim = Some(trim);
+        self
+    }
+
+    /// Set the deletion policy for the XADD operation
+    pub fn set_deletion_policy(mut self, deletion_policy: StreamDeletionPolicy) -> Self {
+        self.deletion_policy = Some(deletion_policy);
+        self
+    }
+}
+
+impl ToRedisArgs for StreamAddOptions {
+    fn write_redis_args<W>(&self, out: &mut W)
+    where
+        W: ?Sized + RedisWrite,
+    {
+        if self.nomkstream {
+            out.write_arg(b"NOMKSTREAM");
+        }
+        if let Some(strategy) = self.trim.as_ref() {
+            strategy.write_redis_args(out);
+        }
+        if let Some(deletion_policy) = self.deletion_policy.as_ref() {
+            deletion_policy.write_redis_args(out);
+        }
+    }
+}
+
 /// Builder options for [`xautoclaim_options`] command.
 ///
 /// [`xautoclaim_options`]: ../trait.Commands.html#method.xautoclaim_options
@@ -101,6 +281,8 @@ pub struct StreamClaimOptions {
     /// Set `JUSTID` cmd arg. Be advised: the response
     /// type changes with this option.
     justid: bool,
+    /// Set `LASTID <lastid>` cmd arg.
+    lastid: Option<String>,
 }
 
 impl StreamClaimOptions {
@@ -134,6 +316,12 @@ impl StreamClaimOptions {
         self.justid = true;
         self
     }
+
+    /// Set `LASTID <lastid>` cmd arg.
+    pub fn with_lastid(mut self, lastid: impl Into<String>) -> Self {
+        self.lastid = Some(lastid.into());
+        self
+    }
 }
 
 impl ToRedisArgs for StreamClaimOptions {
@@ -159,6 +347,10 @@ impl ToRedisArgs for StreamClaimOptions {
         if self.justid {
             out.write_arg(b"JUSTID");
         }
+        if let Some(ref lastid) = self.lastid {
+            out.write_arg(b"LASTID");
+            lastid.write_redis_args(out);
+        }
     }
 }
 
@@ -450,6 +642,12 @@ pub struct StreamInfoGroup {
     pub pending: usize,
     /// Last ID delivered to this group.
     pub last_delivered_id: String,
+    /// The logical "read counter" of the last entry delivered to group's consumers
+    /// (or `None` if the server does not provide the value).
+    pub entries_read: Option<usize>,
+    /// The number of entries in the stream that are still waiting to be delivered to the
+    /// group's consumers, or a `None` when that number can't be determined.
+    pub lag: Option<usize>,
 }
 
 /// Represents a pending message parsed from [`xpending`] methods.
@@ -640,19 +838,11 @@ impl FromRedisValue for StreamPendingReply {
         } else {
             let mut result = StreamPendingData::default();
 
-            let start_id = start.ok_or_else(|| {
-                Error::new(
-                    ErrorKind::Other,
-                    "IllegalState: Non-zero pending expects start id",
-                )
-            })?;
+            let start_id = start
+                .ok_or_else(|| Error::other("IllegalState: Non-zero pending expects start id"))?;
 
-            let end_id = end.ok_or_else(|| {
-                Error::new(
-                    ErrorKind::Other,
-                    "IllegalState: Non-zero pending expects end id",
-                )
-            })?;
+            let end_id =
+                end.ok_or_else(|| Error::other("IllegalState: Non-zero pending expects end id"))?;
 
             result.count = count;
             result.start_id = start_id;
@@ -781,17 +971,144 @@ impl FromRedisValue for StreamInfoGroupsReply {
             if let Some(v) = &map.get("last-delivered-id") {
                 g.last_delivered_id = from_redis_value(v)?;
             }
+            if let Some(v) = &map.get("entries-read") {
+                g.entries_read = if let Value::Nil = v {
+                    None
+                } else {
+                    Some(from_redis_value(v)?)
+                };
+            }
+            if let Some(v) = &map.get("lag") {
+                g.lag = if let Value::Nil = v {
+                    None
+                } else {
+                    Some(from_redis_value(v)?)
+                };
+            }
             reply.groups.push(g);
         }
         Ok(reply)
     }
 }
 
+/// Deletion policy for stream entries.
+#[derive(Debug, Clone, Default)]
+pub enum StreamDeletionPolicy {
+    /// Preserve existing references to the deleted entries in all consumer groups' PEL.
+    #[default]
+    KeepRef,
+    /// Delete the entry from the stream and from all the consumer groups' PELs.
+    DelRef,
+    /// Delete the entry from the stream and from all the consumer groups' PELs, but only if the entry is acknowledged by all the groups.
+    Acked,
+}
+
+impl ToRedisArgs for StreamDeletionPolicy {
+    fn write_redis_args<W>(&self, out: &mut W)
+    where
+        W: ?Sized + RedisWrite,
+    {
+        match self {
+            StreamDeletionPolicy::KeepRef => out.write_arg(b"KEEPREF"),
+            StreamDeletionPolicy::DelRef => out.write_arg(b"DELREF"),
+            StreamDeletionPolicy::Acked => out.write_arg(b"ACKED"),
+        }
+    }
+}
+
+/// Status codes returned by the `XDELEX` command
+#[cfg(feature = "streams")]
+#[cfg_attr(docsrs, doc(cfg(feature = "streams")))]
+#[derive(Debug, PartialEq, Eq)]
+pub enum XDelExStatusCode {
+    /// No entry with the given id exists in the stream
+    IdNotFound = -1,
+    /// The entry was deleted from the stream
+    Deleted = 1,
+    /// The entry was not deleted because it has either not been delivered to any consumer
+    /// or still has references in the consumer groups' Pending Entries List (PEL)
+    NotDeletedUnacknowledgedOrStillReferenced = 2,
+}
+
+#[cfg(feature = "streams")]
+impl FromRedisValue for XDelExStatusCode {
+    fn from_redis_value(v: &Value) -> RedisResult<Self> {
+        match v {
+            Value::Int(code) => match *code {
+                -1 => Ok(XDelExStatusCode::IdNotFound),
+                1 => Ok(XDelExStatusCode::Deleted),
+                2 => Ok(XDelExStatusCode::NotDeletedUnacknowledgedOrStillReferenced),
+                _ => Err(RedisError::from((
+                    ErrorKind::TypeError,
+                    "Invalid XDelExStatusCode status code",
+                    format!("Unknown status code: {code}"),
+                ))),
+            },
+            _ => Err(RedisError::from((
+                ErrorKind::TypeError,
+                "Response type not XDelExStatusCode compatible",
+            ))),
+        }
+    }
+}
+
+/// Status codes returned by the `XACKDEL` command
+#[cfg(feature = "streams")]
+#[cfg_attr(docsrs, doc(cfg(feature = "streams")))]
+#[derive(Debug, PartialEq, Eq)]
+pub enum XAckDelStatusCode {
+    /// No entry with the given id exists in the stream
+    IdNotFound = -1,
+    /// The entry was acknowledged and deleted from the stream
+    AcknowledgedAndDeleted = 1,
+    /// The entry was acknowledged but not deleted because it has references in the consumer groups' Pending Entries List (PEL)
+    AcknowledgedNotDeletedStillReferenced = 2,
+}
+
+#[cfg(feature = "streams")]
+impl FromRedisValue for XAckDelStatusCode {
+    fn from_redis_value(v: &Value) -> RedisResult<Self> {
+        match v {
+            Value::Int(code) => match *code {
+                -1 => Ok(XAckDelStatusCode::IdNotFound),
+                1 => Ok(XAckDelStatusCode::AcknowledgedAndDeleted),
+                2 => Ok(XAckDelStatusCode::AcknowledgedNotDeletedStillReferenced),
+                _ => Err(RedisError::from((
+                    ErrorKind::TypeError,
+                    "Invalid XAckDelStatusCode status code",
+                    format!("Unknown status code: {code}"),
+                ))),
+            },
+            _ => Err(RedisError::from((
+                ErrorKind::TypeError,
+                "Response type not XAckDelStatusCode compatible",
+            ))),
+        }
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
 
-    mod stream_auto_claim_options {
+    fn assert_command_eq(object: impl ToRedisArgs, expected: &[u8]) {
+        let mut out: Vec<Vec<u8>> = Vec::new();
+
+        object.write_redis_args(&mut out);
+
+        let mut cmd: Vec<u8> = Vec::new();
+
+        out.iter_mut().for_each(|item| {
+            cmd.append(item);
+            cmd.push(b' ');
+        });
+
+        cmd.pop();
+
+        assert_eq!(cmd, expected);
+    }
+
+    mod stream_auto_claim_reply {
         use super::*;
         use crate::Value;
 
@@ -960,4 +1277,70 @@ mod tests {
             assert_eq!(reply.deleted_ids.len(), 0);
         }
     }
+
+    mod stream_trim_options {
+        use super::*;
+
+        #[test]
+        fn maxlen_trim() {
+            let options = StreamTrimOptions::maxlen(StreamTrimmingMode::Approx, 10);
+
+            assert_command_eq(options, b"MAXLEN ~ 10");
+        }
+
+        #[test]
+        fn maxlen_exact_trim() {
+            let options = StreamTrimOptions::maxlen(StreamTrimmingMode::Exact, 10);
+
+            assert_command_eq(options, b"MAXLEN = 10");
+        }
+
+        #[test]
+        fn maxlen_trim_limit() {
+            let options = StreamTrimOptions::maxlen(StreamTrimmingMode::Approx, 10).limit(5);
+
+            assert_command_eq(options, b"MAXLEN ~ 10 LIMIT 5");
+        }
+        #[test]
+        fn minid_trim_limit() {
+            let options = StreamTrimOptions::minid(StreamTrimmingMode::Exact, "123456-7").limit(5);
+
+            assert_command_eq(options, b"MINID = 123456-7 LIMIT 5");
+        }
+    }
+
+    mod stream_add_options {
+        use super::*;
+
+        #[test]
+        fn the_default() {
+            let options = StreamAddOptions::default();
+
+            assert_command_eq(options, b"");
+        }
+
+        #[test]
+        fn with_maxlen_trim() {
+            let options = StreamAddOptions::default()
+                .trim(StreamTrimStrategy::maxlen(StreamTrimmingMode::Exact, 10));
+
+            assert_command_eq(options, b"MAXLEN = 10");
+        }
+
+        #[test]
+        fn with_nomkstream() {
+            let options = StreamAddOptions::default().nomkstream();
+
+            assert_command_eq(options, b"NOMKSTREAM");
+        }
+
+        #[test]
+        fn with_nomkstream_and_maxlen_trim() {
+            let options = StreamAddOptions::default()
+                .nomkstream()
+                .trim(StreamTrimStrategy::maxlen(StreamTrimmingMode::Exact, 10));
+
+            assert_command_eq(options, b"NOMKSTREAM MAXLEN = 10");
+        }
+    }
 }
