@@ -31,10 +31,14 @@ pub fn current_thread_runtime() -> tokio::runtime::Runtime {
 }
 
 #[cfg(feature = "aio")]
+#[derive(Clone, Copy)]
 pub enum RuntimeType {
+    #[cfg(feature = "tokio-comp")]
     Tokio,
     #[cfg(feature = "async-std-comp")]
     AsyncStd,
+    #[cfg(feature = "smol-comp")]
+    Smol,
 }
 
 #[cfg(feature = "aio")]
@@ -67,14 +71,17 @@ where
     let f = futures_util::FutureExt::fuse(f);
     futures::pin_mut!(f, check_future);
 
+    let f = async move {
+        futures::select! {res = f => res, err = check_future => err}
+    };
+
     let res = match runtime {
-        RuntimeType::Tokio => current_thread_runtime().block_on(async {
-            futures::select! {res = f => res, err = check_future => err}
-        }),
+        #[cfg(feature = "tokio-comp")]
+        RuntimeType::Tokio => block_on_all_using_tokio(f),
         #[cfg(feature = "async-std-comp")]
-        RuntimeType::AsyncStd => block_on_all_using_async_std(async move {
-            futures::select! {res = f => res, err = check_future => err}
-        }),
+        RuntimeType::AsyncStd => block_on_all_using_async_std(f),
+        #[cfg(feature = "smol-comp")]
+        RuntimeType::Smol => block_on_all_using_smol(f),
     };
 
     let _ = panic::take_hook();
@@ -87,31 +94,66 @@ where
 
 #[cfg(feature = "aio")]
 #[rstest::rstest]
-#[case::tokio(RuntimeType::Tokio)]
+#[cfg_attr(feature = "tokio-comp", case::tokio(RuntimeType::Tokio))]
 #[cfg_attr(feature = "async-std-comp", case::async_std(RuntimeType::AsyncStd))]
+#[cfg_attr(feature = "smol-comp", case::smol(RuntimeType::Smol))]
 #[should_panic(expected = "Internal thread panicked")]
 fn test_block_on_all_panics_from_spawns(#[case] runtime: RuntimeType) {
+    use std::sync::{atomic::AtomicBool, Arc};
+
+    let slept = Arc::new(AtomicBool::new(false));
+    let slept_clone = slept.clone();
     let _ = block_on_all(
         async {
-            spawn(async {
+            spawn(async move {
                 futures_time::task::sleep(futures_time::time::Duration::from_millis(1)).await;
+                slept_clone.store(true, std::sync::atomic::Ordering::Relaxed);
                 panic!("As it should");
             });
-            futures_time::task::sleep(futures_time::time::Duration::from_millis(10)).await;
+
+            loop {
+                futures_time::task::sleep(futures_time::time::Duration::from_millis(2)).await;
+                if slept.load(std::sync::atomic::Ordering::Relaxed) {
+                    break;
+                }
+            }
+
             Ok(())
         },
         runtime,
     );
 }
 
+#[cfg(feature = "tokio-comp")]
+fn block_on_all_using_tokio<F>(f: F) -> F::Output
+where
+    F: Future,
+{
+    #[cfg(any(feature = "async-std-comp", feature = "smol-comp"))]
+    redis::aio::prefer_tokio().unwrap();
+    current_thread_runtime().block_on(f)
+}
+
 #[cfg(feature = "async-std-comp")]
 fn block_on_all_using_async_std<F>(f: F) -> F::Output
 where
     F: Future,
 {
+    #[cfg(any(feature = "tokio-comp", feature = "smol-comp"))]
+    redis::aio::prefer_async_std().unwrap();
     async_std::task::block_on(f)
 }
 
+#[cfg(feature = "smol-comp")]
+fn block_on_all_using_smol<F>(f: F) -> F::Output
+where
+    F: Future,
+{
+    #[cfg(any(feature = "tokio-comp", feature = "async-std-comp"))]
+    redis::aio::prefer_smol().unwrap();
+    smol::block_on(f)
+}
+
 #[cfg(any(feature = "cluster", feature = "cluster-async"))]
 mod cluster;
 
@@ -245,12 +287,6 @@ impl TestContext {
         self.client.get_connection().unwrap()
     }
 
-    #[cfg(feature = "aio")]
-    #[allow(deprecated)]
-    pub async fn deprecated_async_connection(&self) -> RedisResult<redis::aio::Connection> {
-        self.client.get_async_connection().await
-    }
-
     #[cfg(feature = "aio")]
     pub async fn async_connection(&self) -> RedisResult<redis::aio::MultiplexedConnection> {
         self.client.get_multiplexed_async_connection().await
@@ -349,7 +385,7 @@ where
             Ok(())
         }
         Value::Set(ref values) => encode_iter(values, writer, "~"),
-        Value::Double(val) => write!(writer, ",{}\r\n", val),
+        Value::Double(val) => write!(writer, ",{val}\r\n"),
         Value::Boolean(v) => {
             if v {
                 write!(writer, "#t\r\n")
@@ -364,7 +400,7 @@ where
             // format is always 3 bytes
             write!(writer, "={}\r\n{}:{}\r\n", 3 + text.len(), format, text)
         }
-        Value::BigNumber(ref val) => write!(writer, "({}\r\n", val),
+        Value::BigNumber(ref val) => write!(writer, "({val}\r\n"),
         Value::Push { ref kind, ref data } => {
             write!(writer, ">{}\r\n+{kind}\r\n", data.len() + 1)?;
             for val in data.iter() {
@@ -405,6 +441,28 @@ pub fn is_version(expected_major_minor: (u16, u16), version: Version) -> bool {
         || (expected_major_minor.0 == version.0 && expected_major_minor.1 <= version.1)
 }
 
+// Redis version constants for version-gated tests
+pub const REDIS_VERSION_CE_8_0: Version = (8, 0, 0);
+pub const REDIS_VERSION_CE_8_2: Version = (8, 1, 240);
+
+/// Macro to run tests only if the Redis version meets the minimum requirement.
+/// If the version is insufficient, the test is skipped with a message.
+#[macro_export]
+macro_rules! run_test_if_version_supported {
+    ($minimum_required_version:expr) => {{
+        let ctx = $crate::support::TestContext::new();
+        let redis_version = ctx.get_version();
+
+        if redis_version < *$minimum_required_version {
+            eprintln!("Skipping the test because the current version of Redis {:?} doesn't match the minimum required version {:?}.",
+            redis_version, $minimum_required_version);
+            return;
+        }
+
+        ctx
+    }};
+}
+
 #[cfg(feature = "tls-rustls")]
 fn load_certs_from_file(tls_file_paths: &TlsFilePaths) -> TlsCertificates {
     let ca_file = File::open(&tls_file_paths.ca_crt).expect("Cannot open CA cert file");
